@@ -5,7 +5,7 @@ use warnings;
 use 5.010;
 use parent qw/autobox/;
 
-our $VERSION = "1.029";
+our $VERSION = "1.030";
 
 =head1 NAME
 
@@ -38,12 +38,12 @@ particular when the values are hashrefs or objects.
     $book_locations->filter(); # true values
     $books->filter(sub { $_->is_in_library($library) });
     $book_names->filter( qr/lord/i );
-    $book_types->filter("scifi");
-    $book_types->filter({ fantasy => 1, scifi => 1 }); # hash key exists
+    $book_genres->filter("scifi");
+    $book_genres->filter({ fantasy => 1, scifi => 1 }); # hash key exists
 
     # order (like a more succinct sort)
-    $book_types->order;
-    $book_types->order("desc");
+    $book_genres->order;
+    $book_genres->order("desc");
     $book_prices->order([ "num", "desc" ]);
     $books->order([ sub { $_->{price} }, "desc", "num" ]);
     $log_lines->order([ num => qr/pid: "(\d+)"/ ]);
@@ -52,6 +52,11 @@ particular when the values are hashrefs or objects.
         sub { $_->{name} },                    # then name
     );
 
+    # group (aggregate) array into hash
+    $book_genres->group;       # "Sci-fi" => "Sci-fi"
+    $book_genres->group_count; # "Sci-fi" => 3
+    $book_genres->group_array; # "Sci-fi" => [ "Sci-fi", "Sci-fi", "Sci-fi"]
+
     # Flatten arrayrefs-of-arrayrefs
     $authors->map_by("books") # ->books returns an arrayref
     # [ [ $book1, $book2 ], [ $book3 ] ]
@@ -207,6 +212,18 @@ and hashrefs.
 
 =item
 
+@array->group()
+
+=item
+
+@array->group_count()
+
+=item
+
+@array->group_array()
+
+=item
+
 @array->flat()
 
 =item
@@ -647,7 +664,7 @@ The value of join("", @captured_groups) are used in the comparison (@captured_gr
     ->order_by(age => [ sub { int($_) }, "num" ])
 
     # compare int( $a->age_by_interval(10) )
-    ->order_by([ age_by_interval => 10 ] => [ sub { int($_) }, "num" ]) 
+    ->order_by([ age_by_interval => 10 ] => [ sub { int($_) }, "num" ])
     # compare uc( $a->name_with_title($title) )
     ->order_by([ name_with_title => $title ], sub { uc($_) })
 
@@ -932,8 +949,8 @@ L</Sorting using order and order_by> for details about how these work.
 
 Examples:
 
-    @book_types->order;
-    @book_types->order("desc");
+    @book_genres->order;
+    @book_genres->order("desc");
     @book_prices->order([ "num", "desc" ]);
     @books->order([ sub { $_->{price} }, "desc", "num" ]);
     @log_lines->order([ num => qr/pid: "(\d+)"/ ]);
@@ -963,6 +980,148 @@ sub order {
 
 
 
+=head2 @array->group($value_subref = object) : %key_value | %$key_value
+
+Group the @array items into a hashref with the items as keys.
+
+The default $value_subref puts each object in the list as the hash
+value. If the key is repeated, the value is overwritten with the last
+object.
+
+Example:
+
+    my $title_book = $book_titles->group;
+    # {
+    #     "Leviathan Wakes"       => "Leviathan Wakes",
+    #     "Caliban's War"         => "Caliban's War",
+    #     "The Tree-Body Problem" => "The Tree-Body Problem",
+    #     "The Name of the Wind"  => "The Name of the Wind",
+    # },
+
+=head3 The $value_subref
+
+For simple cases of just grouping a single key to a single value, the
+$value_subref is straightforward to use.
+
+The hash key is the array item. The hash value is whatever is returned
+from
+
+    my $new_value = $value_sub->($current_value, $object, $key);
+
+=over 4
+
+=item
+
+C<$current> value is the current hash value for this key (or undef if
+the first one).
+
+=item
+
+C<$object> is the current item in the list. The current $_ is also set
+to this.
+
+=item
+
+C<$key> is the array item.
+
+=back
+
+See also: C<-E<gt>group_by>.
+
+=cut
+
+sub __core_group {
+    my( $name, $array, $value_sub ) = @_;
+    @$array or return wantarray ? () : { };
+
+    my %key_value;
+    for my $item (@$array) {
+        my $key = $item;
+
+        my $current_value = $key_value{ $key };
+        local $_ = $item;
+        my $new_value = $value_sub->($current_value, $item, $key);
+
+        $key_value{ $key } = $new_value;
+    }
+
+    return wantarray ? %key_value : \%key_value;
+}
+
+sub group {
+    my $array = shift;
+    my ($value_sub) = _normalized_accessor_args_subref(@_);
+
+    $value_sub //= sub { $_ };
+    ref($value_sub) eq "CODE"
+        or Carp::croak("group(\$value_sub): \$value_sub ($value_sub) is not a sub ref");
+
+    return __core_group("group", $array, $value_sub);
+}
+
+
+
+=head2 @array->group_count : %key_count | %$key_count
+
+Just like C<group>, but the hash values are the the number of
+instances each item occurs in the list.
+
+Example:
+
+    $book_genres->group_count;
+    # {
+    #     "Sci-fi"  => 3,
+    #     "Fantasy" => 1,
+    # },
+
+There are three books counted for the "Sci-fi" key.
+
+=cut
+
+sub group_count {
+    my $array = shift;
+
+    my $value_sub = sub {
+        my $count = shift // 0; return ++$count;
+    };
+
+    return __core_group("group_count", $array, $value_sub);
+}
+
+
+
+
+=head2 @array->group_array : %key_objects | %$key_objects
+
+Just like C<group>, but the hash values are arrayrefs containing those
+same array items.
+
+Example:
+
+    $book_genres->group_array;
+    # {
+    #     "Sci-fi"  => [ "Sci-fi", "Sci-fi", "Sci-fi" ],
+    #     "Fantasy" => [ "Fantasy" ],
+    # },
+
+The three Sci-fi genres are collected under the Sci-fi key.
+
+=cut
+
+sub group_array {
+    my $array = shift;
+
+    my $value_sub = sub {
+        my $array = shift // [];
+        push( @$array, $_ );
+        return $array;
+    };
+
+    return __core_group("group_array", $array, $value_sub);
+}
+
+
+
 =head2 @array->flat() : @array | @$array
 
 Return a (one level) flattened array, assuming the array items
@@ -1352,10 +1511,8 @@ Example:
 
 =head3 The $value_subref
 
-This is a bit tricky to use, so the most common thing would probably
-be to use one of the more specific group_by-methods (see below). It
-should be capable enough to achieve what you need though, so here's
-how it works:
+For simple cases of just grouping a single key to a single value, the
+$value_subref is straightforward to use.
 
 The hash key is whatever is returned from C<$object-E<gt>$accessor>.
 
@@ -1363,8 +1520,6 @@ The hash value is whatever is returned from
 
     my $new_value = $value_sub->($current_value, $object, $key);
 
-where:
-
 =over 4
 
 =item
@@ -1381,6 +1536,18 @@ C<$key> is the key returned by $object->$accessor(@$args)
 
 =back
 
+A simple example would be to group by the accessor, but instead of the
+object used as the value you want to look up an attribute on each
+object:
+
+    my $book_id__author = $books->group_by("id", sub { $_->author });
+    # keys: book id; values: author
+
+If you want to create an aggregate value the $value_subref can be a
+bit tricky to use, so the most common thing would probably be to use
+one of the more specific group_by-methods (see below). It should be
+capable enough to achieve what you need though.
+
 =cut
 
 sub __core_group_by {
